/*
 * MammothCopyView.java
 */

package mammothcopy;

import org.jdesktop.application.Action;
import org.jdesktop.application.SingleFrameApplication;
import org.jdesktop.application.FrameView;
import java.util.Date;
import java.text.DateFormat;
import java.util.Locale;
import java.io.*;
import javax.swing.*;
import java.net.*;

import net.sbbi.upnp.messages.*;
import net.sbbi.upnp.impls.InternetGatewayDevice;

/**
 * The application's main frame.
 */
public class MammothCopyView extends FrameView {
    public static boolean AppRunning;
    static String trgt;

    // UPnP stuff
    InternetGatewayDevice IGD;
    static final int TIMEOUT_UPNPSEARCH	    = 1500;

    // these could go in a settings place
    static final int SETTINGS_CHUNKSIZE = 1024*1024;
    static int loglevel;


    // cross-thread stuff
    static String logline, mystatus, partnerstatus;
    static String pbMain, pbSub;
    static int pbvMain, pbvSub;

    static final int LEVEL_DEBUG =	50;
    static final int LEVEL_NOTICE =	40;
    static final int LEVEL_WARNING =	30;
    static final int LEVEL_ERROR =	20;
    static final int LEVEL_NONE =	 0;

    // Thread to update the UI from other threads.
    class Logger extends Thread {
        @Override public void run() {
            while (AppRunning) {
                try {    Thread.sleep(100); } catch (InterruptedException e) {}
                if (!logline.equals("")) {
                    Date t = new Date(System.currentTimeMillis());
                    Output.insert(DateFormat.getTimeInstance(DateFormat.MEDIUM, Locale.GERMANY).format(t)+": "+logline+"\n", busyIconIndex);
                    logline = "";
                }
                if (!mystatus.equals("")) {
                    edMyStatus.setText(mystatus);
                    mystatus = "";
                }
                if (!partnerstatus.equals("")) {
                    edPartnerStatus.setText(partnerstatus);
                    partnerstatus = "";
                }
            }
        }
    }

    class UI extends Thread {
        @Override public void run() {
            while (AppRunning) {
                try {    Thread.sleep(100); } catch (InterruptedException e) {}
                if (TheSocket != null && TheSocket.connected) {
                    // when connected, disable connect and start buttons,
                    // and enable cancel or SEND buttons
                    btnStart.setEnabled(false);
                    btnConnect.setEnabled(false);
                    btnGo.setEnabled(true);
                    btnCancel.setEnabled(true);
		    lblConnected.setText("Connected.");
                } else {
                    btnStart.setEnabled(true);
                    btnConnect.setEnabled(true);
                    btnGo.setEnabled(false);
                    btnCancel.setEnabled(false);
 		    lblConnected.setText("Not connected.");
                }

                // set button
                if (server != null && server.ServerRunning == true) {
                    btnStart.setText("Stop");
                } else {
                    btnStart.setText("Start");
                }

		if (ProgressBar.getValue() != pbvMain) {
		    ProgressBar.setValue(pbvMain);
		}
		if (!ProgressBar.getString().equals(pbMain)) {
		    ProgressBar.setString(pbMain);
		}
		if (SubProgressBar.getValue() != pbvSub) {
		    SubProgressBar.setValue(pbvSub);
		}
		if (!SubProgressBar.getString().equals(pbSub)) {
		    SubProgressBar.setString(pbSub);
		}

		// fetch the target string
		trgt = edTarget.getText();
            }
        }
    }

    public static void setMyStatus(String s) {
        mystatus = s;
    }
    public static void setPartnerStatus(String s) {
        partnerstatus = s;
    }
    public static void SetProgressBar(String s, int value) {
	pbvMain = value;
	pbMain = s;
    }
    public static void SetSubProgressBar(String s, int value) {
	pbvSub = value;
	pbSub = s;
    }


    public static void Log(String s, int level) {
	if (level <= loglevel) {
	    if (logline.equals("")) {
		logline = s;
	    } else {
		try {
		    Thread.sleep(10);
		    Log(s, level);
		}
		catch (InterruptedException e) {}
	    }
	}
    }

    private String VERSION = "0.1";

    public MammothCopyView(SingleFrameApplication app) {
        super(app);

	loglevel = LEVEL_DEBUG;

        initComponents();

        AppRunning = true;
        logline = "";
        mystatus = "";
        partnerstatus = "";
	IGD = null;

        // our progress bar works with percentages
        ProgressBar.setMaximum(100);
        ProgressBar.setMinimum(0);
        ProgressBar.setValue(0);

        // default, we cannot start
        btnGo.setEnabled(false);

        // Find out our own IP address
        // When using UPnP, this will be our routable internet address
        try {
            InetAddress myIP = InetAddress.getLocalHost();
            edMyIP.setText(myIP.getHostAddress());
            Log("Local IP address set to: "+myIP.getHostAddress(), MammothCopyView.LEVEL_NOTICE);

            // Use a default port. No way of saving settings yet
        } catch (UnknownHostException e) {
            Log("Could not find my IP address.", MammothCopyView.LEVEL_WARNING);
            edMyIP.setText("0.0.0.0");
        }

        edMyPort.setText("1976");
        edPartnerPort.setText("1976");

        l = new Logger();
        l.start();

        u = new UI();
        u.start();

	ProgressBar.setMinimum(0);
	ProgressBar.setMaximum(100);
	SubProgressBar.setMinimum(0);
	SubProgressBar.setMaximum(100);

        Log("MammothCopy v"+VERSION+" started.", MammothCopyView.LEVEL_NOTICE);



//	// TEST GROUNDS --------------------------------------------------------------------------------------------------------------------------


/*	ProgressBar.setMinimum(0);
	ProgressBar.setMaximum(100);
	ProgressBar.setValue(34);
	ProgressBar.setString("34/100");
*/
//	try {
//	    InternetGatewayDevice[] IGDs = InternetGatewayDevice.getDevices( 1500 );
//	    if (IGDs != null) {
//		Log("Found a Internet Gateway Device (IGD): "+IGDs[0].getIGDRootDevice().getModelName(), MammothCopyView.LEVEL_DEBUG);
//		String localHostIP = InetAddress.getLocalHost().getHostAddress();
//
//		Log("Mapping local IP "+localHostIP, MammothCopyView.LEVEL_DEBUG);
//
//		boolean mapped = IGDs[0].addPortMapping( "Some mapping description",
//		null, 9090, 9090, localHostIP, 0, "TCP" );
//
//		if (mapped) {
//		    Log("Yay! Mapping was made!", MammothCopyView.LEVEL_DEBUG);
//		    boolean unmapped = IGDs[0].deletePortMapping( null, 9090, "TCP" );
//		    if ( unmapped ) {
//			Log( "Port 9090 unmapped" , MammothCopyView.LEVEL_DEBUG);
//		    } else {
//			Log("Not unmapped.", MammothCopyView.LEVEL_DEBUG);
//		    }
//
//		} else {
//		    Log("Not mapped.", MammothCopyView.LEVEL_DEBUG);
//		}
//
//
//	      }
//
//
//	} catch (Exception e) {
//
//	}


    }

    private void PunchHole() {
	try {
	    if (IGD == null) {
		Log("Searching for internet gateway devices (timeout="+TIMEOUT_UPNPSEARCH, LEVEL_NOTICE);
		InternetGatewayDevice[] IGDs = InternetGatewayDevice.getDevices(TIMEOUT_UPNPSEARCH);
		if (IGDs != null) {
		    // we found some, just use the first one
		    if (IGDs.length > 1) {
			Log("Found more than one IGD. Using the first one.", LEVEL_NOTICE);
		    }
		    IGD = IGDs[0];

		    int port = Integer.parseInt(edMyPort.getText());

		    try {
			// try to get a outside port
			boolean mapped = IGD.addPortMapping("MammothCopy", null, port, port, edMyIP.getText(), 0, "TCP");

			if (mapped) {
			    // mapping OK
			    String out_ip = IGD.getExternalIPAddress();
			    Log("Mapping made, outside IP set to "+out_ip, LEVEL_NOTICE);
			    edMyIP.setText(out_ip);
			} else {
			    // mapping not OK
			    Log("Couldn't map port through IGD, already taken.", LEVEL_WARNING);
			}
		    } catch (UPNPResponseException e) {
			Log("Couldn't map port through IGD: "+e.getDetailErrorDescription(), LEVEL_WARNING);
		    }
		}
	    }
	} catch (IOException e) {
	    Log("Couldn't search for IGDs, no hole punched, using local IP and port.", MammothCopyView.LEVEL_WARNING);
	}
    }
    private void UnpunchHole() {
	try {
	    if (IGD != null && IGD.deletePortMapping(null, 1976, "TCP")) {
		Log("Mapping unmade.", LEVEL_NOTICE);
	    } else {
	       Log("Could not unmap.", LEVEL_NOTICE);
	    }
	} catch (IOException e) {
	    Log("Couldn't unmap.", MammothCopyView.LEVEL_WARNING);
	} catch (UPNPResponseException e) {
	    Log("Couldn't unmap: "+e.getDetailErrorDescription(), LEVEL_WARNING);
	}
    }

    @Action
    public void showAboutBox() {
        if (aboutBox == null) {
            JFrame mainFrame = MammothCopyApp.getApplication().getMainFrame();
            aboutBox = new MammothCopyAboutBox(mainFrame);
            aboutBox.setLocationRelativeTo(mainFrame);
        }
        MammothCopyApp.getApplication().show(aboutBox);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        mainPanel = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        jScrollPane1 = new javax.swing.JScrollPane();
        Output = new javax.swing.JTextArea();
        btnChooseFile = new javax.swing.JButton();
        ProgressBar = new javax.swing.JProgressBar();
        edSource = new javax.swing.JTextField();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        btnGo = new javax.swing.JButton();
        jPanel1 = new javax.swing.JPanel();
        jLabel4 = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        edPartnerIP = new javax.swing.JTextField();
        edPartnerPort = new javax.swing.JTextField();
        jLabel8 = new javax.swing.JLabel();
        edPartnerStatus = new javax.swing.JTextField();
        btnConnect = new javax.swing.JButton();
        jPanel2 = new javax.swing.JPanel();
        jLabel6 = new javax.swing.JLabel();
        jLabel7 = new javax.swing.JLabel();
        edMyIP = new javax.swing.JTextField();
        edMyPort = new javax.swing.JTextField();
        jLabel9 = new javax.swing.JLabel();
        edMyStatus = new javax.swing.JTextField();
        btnStart = new javax.swing.JButton();
        jLabel10 = new javax.swing.JLabel();
        edTarget = new javax.swing.JTextField();
        btnChooseFile1 = new javax.swing.JButton();
        btnCancel = new javax.swing.JButton();
        lblConnected = new javax.swing.JLabel();
        SubProgressBar = new javax.swing.JProgressBar();
        menuBar = new javax.swing.JMenuBar();
        javax.swing.JMenu fileMenu = new javax.swing.JMenu();
        javax.swing.JMenuItem exitMenuItem = new javax.swing.JMenuItem();
        javax.swing.JMenu helpMenu = new javax.swing.JMenu();
        javax.swing.JMenuItem aboutMenuItem = new javax.swing.JMenuItem();

        mainPanel.setName("mainPanel"); // NOI18N
        mainPanel.setPreferredSize(new java.awt.Dimension(1000, 570));

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(mammothcopy.MammothCopyApp.class).getContext().getResourceMap(MammothCopyView.class);
        jLabel1.setText(resourceMap.getString("jLabel1.text")); // NOI18N
        jLabel1.setName("jLabel1"); // NOI18N

        jScrollPane1.setName("jScrollPane1"); // NOI18N

        Output.setColumns(20);
        Output.setRows(5);
        Output.setName("Output"); // NOI18N
        jScrollPane1.setViewportView(Output);

        btnChooseFile.setText(resourceMap.getString("btnChooseFile.text")); // NOI18N
        btnChooseFile.setName("btnChooseFile"); // NOI18N
        btnChooseFile.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnChooseFileActionPerformed(evt);
            }
        });

        ProgressBar.setName("ProgressBar"); // NOI18N
        ProgressBar.setStringPainted(true);

        edSource.setText(resourceMap.getString("edSource.text")); // NOI18N
        edSource.setName("edSource"); // NOI18N

        jLabel2.setText(resourceMap.getString("jLabel2.text")); // NOI18N
        jLabel2.setName("jLabel2"); // NOI18N

        jLabel3.setText(resourceMap.getString("jLabel3.text")); // NOI18N
        jLabel3.setName("jLabel3"); // NOI18N

        btnGo.setFont(resourceMap.getFont("btnGo.font")); // NOI18N
        btnGo.setIcon(resourceMap.getIcon("btnGo.icon")); // NOI18N
        btnGo.setText(resourceMap.getString("btnGo.text")); // NOI18N
        btnGo.setName("btnGo"); // NOI18N
        btnGo.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnGoActionPerformed(evt);
            }
        });

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder(resourceMap.getString("jPanel1.border.title"))); // NOI18N
        jPanel1.setName("jPanel1"); // NOI18N

        jLabel4.setText(resourceMap.getString("jLabel4.text")); // NOI18N
        jLabel4.setName("jLabel4"); // NOI18N

        jLabel5.setText(resourceMap.getString("jLabel5.text")); // NOI18N
        jLabel5.setName("jLabel5"); // NOI18N

        edPartnerIP.setText(resourceMap.getString("edPartnerIP.text")); // NOI18N
        edPartnerIP.setName("edPartnerIP"); // NOI18N

        edPartnerPort.setText(resourceMap.getString("edPartnerPort.text")); // NOI18N
        edPartnerPort.setName("edPartnerPort"); // NOI18N

        jLabel8.setText(resourceMap.getString("jLabel8.text")); // NOI18N
        jLabel8.setName("jLabel8"); // NOI18N

        edPartnerStatus.setEditable(false);
        edPartnerStatus.setName("edPartnerStatus"); // NOI18N

        btnConnect.setText(resourceMap.getString("btnConnect.text")); // NOI18N
        btnConnect.setName("btnConnect"); // NOI18N
        btnConnect.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnConnectActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jLabel4)
                    .addComponent(jLabel5))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(edPartnerPort)
                    .addComponent(edPartnerIP, javax.swing.GroupLayout.DEFAULT_SIZE, 209, Short.MAX_VALUE))
                .addGap(18, 18, 18)
                .addComponent(jLabel8)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(edPartnerStatus, javax.swing.GroupLayout.DEFAULT_SIZE, 269, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(btnConnect, javax.swing.GroupLayout.PREFERRED_SIZE, 117, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel4)
                    .addComponent(edPartnerIP, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel8)
                    .addComponent(edPartnerStatus, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(btnConnect))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel5)
                    .addComponent(edPartnerPort, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jPanel2.setBorder(javax.swing.BorderFactory.createTitledBorder(resourceMap.getString("jPanel2.border.title"))); // NOI18N
        jPanel2.setName("jPanel2"); // NOI18N

        jLabel6.setText(resourceMap.getString("jLabel6.text")); // NOI18N
        jLabel6.setName("jLabel6"); // NOI18N

        jLabel7.setText(resourceMap.getString("jLabel7.text")); // NOI18N
        jLabel7.setName("jLabel7"); // NOI18N

        edMyIP.setName("edMyIP"); // NOI18N

        edMyPort.setText(resourceMap.getString("edMyPort.text")); // NOI18N
        edMyPort.setName("edMyPort"); // NOI18N

        jLabel9.setText(resourceMap.getString("jLabel9.text")); // NOI18N
        jLabel9.setName("jLabel9"); // NOI18N

        edMyStatus.setEditable(false);
        edMyStatus.setName("edMyStatus"); // NOI18N

        btnStart.setText(resourceMap.getString("btnStart.text")); // NOI18N
        btnStart.setName("btnStart"); // NOI18N
        btnStart.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnStartActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jLabel6)
                    .addComponent(jLabel7))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(edMyIP)
                    .addComponent(edMyPort, javax.swing.GroupLayout.DEFAULT_SIZE, 205, Short.MAX_VALUE))
                .addGap(18, 18, 18)
                .addComponent(jLabel9)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(edMyStatus, javax.swing.GroupLayout.DEFAULT_SIZE, 273, Short.MAX_VALUE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(btnStart, javax.swing.GroupLayout.PREFERRED_SIZE, 117, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel6)
                    .addComponent(edMyIP, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel9)
                    .addComponent(edMyStatus, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(btnStart))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(edMyPort, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel7))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        jLabel10.setText(resourceMap.getString("jLabel10.text")); // NOI18N
        jLabel10.setName("jLabel10"); // NOI18N

        edTarget.setName("edTarget"); // NOI18N

        btnChooseFile1.setText(resourceMap.getString("btnChooseFile1.text")); // NOI18N
        btnChooseFile1.setName("btnChooseFile1"); // NOI18N
        btnChooseFile1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnChooseFile1ActionPerformed(evt);
            }
        });

        btnCancel.setFont(resourceMap.getFont("btnCancel.font")); // NOI18N
        btnCancel.setText(resourceMap.getString("btnCancel.text")); // NOI18N
        btnCancel.setName("btnCancel"); // NOI18N
        btnCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnCancelActionPerformed(evt);
            }
        });

        lblConnected.setFont(resourceMap.getFont("lblConnected.font")); // NOI18N
        lblConnected.setText(resourceMap.getString("lblConnected.text")); // NOI18N
        lblConnected.setName("lblConnected"); // NOI18N

        SubProgressBar.setName("SubProgressBar"); // NOI18N
        SubProgressBar.setStringPainted(true);

        javax.swing.GroupLayout mainPanelLayout = new javax.swing.GroupLayout(mainPanel);
        mainPanel.setLayout(mainPanelLayout);
        mainPanelLayout.setHorizontalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(mainPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 785, Short.MAX_VALUE)
                    .addComponent(SubProgressBar, javax.swing.GroupLayout.DEFAULT_SIZE, 785, Short.MAX_VALUE)
                    .addComponent(ProgressBar, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 785, Short.MAX_VALUE)
                    .addGroup(mainPanelLayout.createSequentialGroup()
                        .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(jLabel10)
                            .addComponent(jLabel2))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(mainPanelLayout.createSequentialGroup()
                                .addComponent(edTarget, javax.swing.GroupLayout.DEFAULT_SIZE, 486, Short.MAX_VALUE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(btnChooseFile1, javax.swing.GroupLayout.PREFERRED_SIZE, 164, javax.swing.GroupLayout.PREFERRED_SIZE))
                            .addGroup(mainPanelLayout.createSequentialGroup()
                                .addComponent(edSource, javax.swing.GroupLayout.DEFAULT_SIZE, 486, Short.MAX_VALUE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(btnChooseFile, javax.swing.GroupLayout.PREFERRED_SIZE, 164, javax.swing.GroupLayout.PREFERRED_SIZE))))
                    .addComponent(jPanel2, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(jPanel1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(jLabel3)
                    .addGroup(mainPanelLayout.createSequentialGroup()
                        .addComponent(btnGo, javax.swing.GroupLayout.PREFERRED_SIZE, 318, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(btnCancel, javax.swing.GroupLayout.PREFERRED_SIZE, 120, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addComponent(lblConnected))
                    .addComponent(jLabel1))
                .addContainerGap())
        );
        mainPanelLayout.setVerticalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(mainPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(edSource, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel2)
                    .addComponent(btnChooseFile))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(edTarget, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(btnChooseFile1, javax.swing.GroupLayout.PREFERRED_SIZE, 25, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel10))
                .addGap(18, 18, 18)
                .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(18, 18, 18)
                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(btnCancel, javax.swing.GroupLayout.DEFAULT_SIZE, 49, Short.MAX_VALUE)
                        .addComponent(lblConnected, javax.swing.GroupLayout.PREFERRED_SIZE, 42, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(btnGo, javax.swing.GroupLayout.PREFERRED_SIZE, 49, Short.MAX_VALUE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jLabel3)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(ProgressBar, javax.swing.GroupLayout.PREFERRED_SIZE, 26, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(SubProgressBar, javax.swing.GroupLayout.PREFERRED_SIZE, 22, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jLabel1)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 164, Short.MAX_VALUE)
                .addContainerGap())
        );

        menuBar.setName("menuBar"); // NOI18N

        fileMenu.setText(resourceMap.getString("fileMenu.text")); // NOI18N
        fileMenu.setName("fileMenu"); // NOI18N

        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(mammothcopy.MammothCopyApp.class).getContext().getActionMap(MammothCopyView.class, this);
        exitMenuItem.setAction(actionMap.get("quit")); // NOI18N
        exitMenuItem.setName("exitMenuItem"); // NOI18N
        fileMenu.add(exitMenuItem);

        menuBar.add(fileMenu);

        helpMenu.setText(resourceMap.getString("helpMenu.text")); // NOI18N
        helpMenu.setName("helpMenu"); // NOI18N

        aboutMenuItem.setAction(actionMap.get("showAboutBox")); // NOI18N
        aboutMenuItem.setName("aboutMenuItem"); // NOI18N
        helpMenu.add(aboutMenuItem);

        menuBar.add(helpMenu);

        setComponent(mainPanel);
        setMenuBar(menuBar);
    }// </editor-fold>//GEN-END:initComponents


    private void btnStartActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnStartActionPerformed
        // START
        //
        // Check and start the 'ME' part

        try {
            // start/stop server
            if (server != null && server.ServerRunning == true) {
                // server runs. stop it
                server.Stop();

		UnpunchHole();
            } else {
		PunchHole();

                // create server
		InetAddress myIP = InetAddress.getLocalHost();
                String port = edMyPort.getText();
                String ip = myIP.getHostAddress();
                server = new MammothServer(Integer.parseInt(port), InetAddress.getByName(ip));

                server.start();
            }
        } catch (UnknownHostException e) {
            Log("Host not found.", MammothCopyView.LEVEL_WARNING);
        } 

    }//GEN-LAST:event_btnStartActionPerformed

    private void btnConnectActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnConnectActionPerformed
        String ip = edPartnerIP.getText();
        int port = Integer.parseInt(edPartnerPort.getText());
        try {

            Socket s = new Socket(ip, port);
            partnerstatus = "Connected to partner "+ip+":"+port;

            Connected(s);
        } catch (UnknownHostException e) {
            Log("Unknown host: "+edPartnerIP.getText(), MammothCopyView.LEVEL_WARNING);
            partnerstatus = e.toString();
        } catch (java.io.IOException e) {
            Log("I/O Error: "+e.toString()+" (tried: Socket("+ip+":"+port+")", MammothCopyView.LEVEL_ERROR);
            partnerstatus = e.toString();
        }
    }//GEN-LAST:event_btnConnectActionPerformed

    private void btnChooseFileActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnChooseFileActionPerformed
                // Choose file/folder to SEND
        JFrame mainFrame = MammothCopyApp.getApplication().getMainFrame();
        final JFileChooser fc = new JFileChooser();

        fc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);

        //In response to a button click:
        int returnVal = fc.showOpenDialog(mainFrame);

        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File selection = fc.getSelectedFile();
            Log("File/folder to SEND selected: "+fc.getName(selection), MammothCopyView.LEVEL_NOTICE);
            edSource.setText(selection.getAbsolutePath());
        }
    }//GEN-LAST:event_btnChooseFileActionPerformed

    private void btnChooseFile1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnChooseFile1ActionPerformed
         // Choose file/folder to SEND
        JFrame mainFrame = MammothCopyApp.getApplication().getMainFrame();
        final JFileChooser fc = new JFileChooser();

        fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);

        //In response to a button click:
        int returnVal = fc.showOpenDialog(mainFrame);

        if (returnVal == JFileChooser.APPROVE_OPTION) {
            File selection = fc.getSelectedFile();
            Log("Folder to RECEIVE selected: "+fc.getName(selection), MammothCopyView.LEVEL_NOTICE);
            edTarget.setText(selection.getAbsolutePath());
        }
    }//GEN-LAST:event_btnChooseFile1ActionPerformed

    private void btnCancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnCancelActionPerformed
        if (TheSocket.connected) {
            // disconnect, when connected
            partnerstatus = "";
	    TheSocket.Cancel();
        }
    }//GEN-LAST:event_btnCancelActionPerformed

    private void btnGoActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnGoActionPerformed
        // The GO! is easy, notify our MammothSocket that it can start
        // the handshake and sending the file(s).
        if (TheSocket.SetFileToSend(edSource.getText()) > 0) {
            TheSocket.Go("md5", SETTINGS_CHUNKSIZE);
	} else {
	    Log("No files found.", MammothCopyView.LEVEL_WARNING);
	}
    }//GEN-LAST:event_btnGoActionPerformed

    public static void Connected(Socket s) {
        TheSocket = new MammothSocket(s);
	TheSocket.setTarget(trgt);
        TheSocket.start();
    }

    static public boolean IsConnected() {
	return TheSocket != null && TheSocket.connected;
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextArea Output;
    private javax.swing.JProgressBar ProgressBar;
    private javax.swing.JProgressBar SubProgressBar;
    private javax.swing.JButton btnCancel;
    private javax.swing.JButton btnChooseFile;
    private javax.swing.JButton btnChooseFile1;
    private javax.swing.JButton btnConnect;
    private javax.swing.JButton btnGo;
    private javax.swing.JButton btnStart;
    private javax.swing.JTextField edMyIP;
    private javax.swing.JTextField edMyPort;
    private javax.swing.JTextField edMyStatus;
    private javax.swing.JTextField edPartnerIP;
    private javax.swing.JTextField edPartnerPort;
    private javax.swing.JTextField edPartnerStatus;
    private javax.swing.JTextField edSource;
    private javax.swing.JTextField edTarget;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JLabel lblConnected;
    private javax.swing.JPanel mainPanel;
    private javax.swing.JMenuBar menuBar;
    // End of variables declaration//GEN-END:variables

//    private final Timer messageTimer;
//    private final Timer busyIconTimer;
//    private final Icon idleIcon;
//    private final Icon[] busyIcons = new Icon[15];
    private int busyIconIndex = 0;

    private JDialog aboutBox;

    // Threads
    MammothServer server;
    Logger l;
    UI u;
    static MammothSocket TheSocket;
}
