/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package mammothcopy;

import java.security.*;
import java.io.*;

/**
 *
 * @author bart
 */
public class MammothReceiver implements MammothRole {
    MammothSocket socket;

    String filename;

    // last chunk of current file
    private int lastchunk = 0;
    
    File target_file = null;		    // current working file
    String target_file_name = "";
    String target_file_hash = "";
    RandomAccessFile target_file_out = null;
    String target_relpath = null;
    long target_size = 0;

    // run the sender role. note that this function runs inside a thread
    // and should not wait. blocking is OK, we cancel anyway when needed
    // it should use blocking reads to avoid a busy-wait loop
    public void Run(MammothSocket _s, MammothTelegram tel) {
	socket = _s;

	// this call is done after the HS REQ is received
	// we start with sending the HS ACK/NAK
	// data[0] is hash function
	// data[1] is chunksize
	try {
	    socket.md = MessageDigest.getInstance(tel.data[0]);
	    socket.chunksize = Integer.parseInt(tel.data[1]);
	    socket.out.println("HS:"+MammothTelegram.PROT_HS_ACK);
	    File t = new File(socket.target);
	    if (!t.isDirectory() || !t.canWrite()) {
		throw new Exception("Target is not a directory, or not writable.");
	    } else {
		// make sure we and with path separator
		if (!socket.target.endsWith(File.separator)) {
		    socket.target = socket.target+File.separator;
		}
	    }

	    // we acked the handshake, we are ready to receive files
	    MammothCopyView.Log("Sent ACK on handshake. Ready to receive files. Hash: "+tel.data[0]+", chunksize: "+tel.data[1], MammothCopyView.LEVEL_DEBUG);
	} catch (Exception e) {
	    MammothCopyView.Log("Could not ACK the handshake: "+e, MammothCopyView.LEVEL_WARNING);
	    socket.out.println("HS:"+MammothTelegram.PROT_HS_NAK);
	    socket.Cancel();
	}

	try {
	    while (socket.connected) {
		// 1. receive RTS and ACK/NAK it
		if (ReceiveRTS()) {
		    MammothCopyView.Log("RTS received, starting chunks.", MammothCopyView.LEVEL_DEBUG);
		    ReceiveChunks();
		    MammothCopyView.Log("Chunks done.", MammothCopyView.LEVEL_DEBUG);
		}
	    }
	} catch (Exception e) {
	    MammothCopyView.Log("Trouble in receiving the file: "+e, MammothCopyView.LEVEL_ERROR);
	    socket.Cancel();
	}

    }

    private boolean ReceiveRTS() throws IOException {
	boolean r = false;
	lastchunk = 0;

	String line = socket.in.readLine();
	MammothTelegram result = new MammothTelegram(line);
	String reply = "RTS:";

	switch (result.status) {
	    case MammothTelegram.PROT_RTS_RTS:
		// we got an RTS, 0=filename, 1=filesize, 2=hash
		String log = "Received RTS for file: "+socket.target+result.data[0];
		MammothFile newfile = new MammothFile(socket.target+result.data[0], socket.md);
		target_relpath = result.data[0];
		target_size = Long.parseLong(result.data[1]);
		target_file_hash = result.data[2];
		long freespace = new File(socket.target).getUsableSpace();
		if (freespace < Long.parseLong(result.data[1])) {
		    reply += MammothTelegram.PROT_RTS_DISKFULL+":"+result.data[0];
		    log += "; disk full: needed "+Generic.toHumanReadable(target_size)+"B, have "+Generic.toHumanReadable(freespace)+"B.";
		} else if (!newfile.exists()) {
		    // check for <filename>.mammoth
		    target_file_name = socket.target+result.data[0];
		    target_file = new File(target_file_name+".mammoth");
		    if (target_file.exists()) {
			lastchunk = GetLastChunk(target_file);
			reply += MammothTelegram.PROT_RTS_PARTDONE+":"+result.data[0]+":"+lastchunk;
			log += "; file is partly done. Last chunk: "+lastchunk;

			target_file_out = new RandomAccessFile(target_file, "rw");			

			// we're ok
			r = true;
		    } else {
			// file doesn't exist and we have write permissions
			reply += MammothTelegram.PROT_RTS_FILENEW+":"+result.data[0];
			log += "; file is new.";

			// The target file might be in a relative directory that doesn exist.
			// Fix that.
			boolean dir_ok = Generic.FixRelativeDirectory(target_file);

			if (dir_ok == true) {
			    target_file_out = new RandomAccessFile(target_file, "rw");

			    // we're ok
			    r = true;
			} else {
			    reply += MammothTelegram.PROT_RTS_NOPERM+":"+result.data[0];
			}
		    }
		} else {
		    if (newfile.hash().equals(result.data[2])) {
			reply += MammothTelegram.PROT_RTS_FILEDONE+":"+result.data[0];
			log += "; file is done, hash matches.";
		    } else {
			reply += MammothTelegram.PROT_RTS_NOMATCH+":"+result.data[0];
			log += "; file is done, but no hash match.";
		    }
		}

		MammothCopyView.Log(log, MammothCopyView.LEVEL_NOTICE);
		socket.out.println(reply);
		break;
	    case MammothTelegram.PROT_EMPTY:
		// empty telegram is a disconnect from the other side
		MammothCopyView.Log("Other side disconnected.", MammothCopyView.LEVEL_NOTICE);
		socket.Cancel();
		break;
	    case MammothTelegram.PROT_ERROR:
	    default:
		MammothCopyView.Log("Received "+result+", while waiting for RTS.", MammothCopyView.LEVEL_WARNING);
		break;
	}


	return r;
    }

    private int GetLastChunk(File f) {
	return ((int) f.length() / socket.chunksize) - 1;	    // -1, chunk numbers are zero-based
    }

    private int GetChunkLength(long chunknumber) {
	int l = 0;

	if (target_size / ((chunknumber+1) * socket.chunksize) >= 1) {
	    // some whole chunk in the middle of the file
	    l = socket.chunksize;
	} else {
	    // remainder of file after N chunks
	    l = (int)(target_size % socket.chunksize);
	}

	return l;
    }

    // pre: the RTS is OK, 'partfile' is set, only call this function
    // after 'ReceiveRTS' has returned true
    private void ReceiveChunks() throws IOException {
	String line;
	MammothTelegram result;
	boolean done = false;

	while (!done && socket.connected) {
	    line = socket.in.readLine();

/*	    byte[] tmp = new byte[1024];
	    int tmpread = socket.sock.getInputStream().read(tmp, 0, 1024);
	    MammothCopyView.Log("Read "+tmpread+" stray bytes.");
*/
	    result = new MammothTelegram(line);
	    byte[] theChunk;

	    switch (result.status) {
		case MammothTelegram.PROT_CHUNK_CHUNK:
		    // we are going to get a good chunk
		    // result.data[0] if relpath, result.data[1] is the chunk number, result.data[2] is the hash
    		    long chunkno = Long.parseLong(result.data[1]);
		    int chunklength = GetChunkLength(chunkno);

		    MammothCopyView.Log("[telegram] Received CHUNK: "+line, MammothCopyView.LEVEL_DEBUG);

		    if (chunklength > 0) {
			theChunk = new byte[chunklength];

			int bytesleft = chunklength;
			int bytesread = 0;
			MammothCopyView.Log("[telegram] Receiving a chunk of "+chunklength+" bytes long.", MammothCopyView.LEVEL_DEBUG);

			BufferedInputStream bis;
			bis = new BufferedInputStream(socket.sock.getInputStream());

			int offset = 0;
			while (bytesleft > 0 && bytesread > -1) {
			    MammothCopyView.Log("[data] Going to read "+bytesleft+" bytes.", MammothCopyView.LEVEL_DEBUG);
			    bytesread = bis.read(theChunk, offset, bytesleft);
			    offset += bytesread;
			    bytesleft -= bytesread;
    			    MammothCopyView.Log("[data] Read "+bytesread+" bytes, "+bytesleft+" bytes left.", MammothCopyView.LEVEL_DEBUG);
			}

			String received_hash = Generic.toHash(theChunk, socket.md);
			String reply = "";
			String log = "";

			if (received_hash.equals(result.data[2])) {
			    // chunk was ok
			    reply = "CHUNK:"+MammothTelegram.PROT_CHUNK_DONE+":"+target_relpath+":"+result.data[1];
			    log = "[data] Chunk #"+chunkno+" received OK: "+received_hash;
			} else {
			    // chunk was not ok, sender decides on re-send
			    reply = "CHUNK:"+MammothTelegram.PROT_CHUNK_NOMATCH+":"+target_relpath+":"+result.data[1];
			    log = "Chunk #"+chunkno+" NOT received OK. Got "+received_hash+", needed "+result.data[2];
			}

			// write the chunk to our current file
			target_file_out.seek(chunkno*socket.chunksize);
			target_file_out.write(theChunk);

			socket.out.println(reply);
			MammothCopyView.Log(log, MammothCopyView.LEVEL_DEBUG);

			if (chunklength < socket.chunksize) {
			    // last chunk is always smaller than chunksize (unless it is zero)
			    done = true;
			}
		    } else if (chunklength == 0) {
			// file is exactly N chunks, we're done
			done = true;
		    }
		    break;
		case MammothTelegram.PROT_EMPTY:
		case MammothTelegram.PROT_ERROR:
		    // other side disconnected.
		    MammothCopyView.Log("Other side disconnected.", MammothCopyView.LEVEL_NOTICE);
		    socket.Cancel();
		    break;
		default:
		    // what the fsck was this?
		    MammothCopyView.Log("Got some nonsense, ignored: "+line, MammothCopyView.LEVEL_ERROR);
		    break;
	    }
	}
	target_file.renameTo(new File(target_file_name));

	MammothFile h = new MammothFile(target_file_name, socket.md);
	if (h.hash().equals(target_file_hash)) {
	    MammothCopyView.Log("File done, hash match: "+target_relpath, MammothCopyView.LEVEL_NOTICE);
	} else {
	    MammothCopyView.Log("File done, no hash match: "+target_relpath, MammothCopyView.LEVEL_WARNING);
	    h.renameTo(new File(target_file_name+".hasherr"));
	}
    }
}
