/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package mammothcopy;

import java.net.*;
import java.io.*;
import java.util.*;

/**
 *
 * @author bart
 */
public class MammothSender implements MammothRole {
    MammothSocket socket;

    static final int SENDER_NOSEND = -1;
    static final int SENDER_STOPTRANSFER = -2;

    // return chunk to start with, do not start if chunk # is -1
    private int SendRTS(MammothFile file) throws IOException {
	int r = SENDER_NOSEND;

	// create relative path
	String relpath = socket.getRelativePath(file);

	String tel = "RTS:"+MammothTelegram.PROT_RTS_RTS+":"+relpath+":"+file.length()+":"+file.hash();
	socket.out.println(tel);

	// wait for the result
	String line = socket.in.readLine();
	MammothTelegram result = new MammothTelegram(line);

	switch (result.status) {
	    case MammothTelegram.PROT_RTS_NOMATCH:
		// the file exists, but there was no hash match:
		// do not copy file, as this might be a different one
		// we don't want to overwrite the target's
		// wedding video.
		MammothCopyView.Log(relpath+": File on target exists and is done, but is no match.", MammothCopyView.LEVEL_NOTICE);
		break;
	    case MammothTelegram.PROT_RTS_DISKFULL:
		// no room to receive the file, do not send.
		MammothCopyView.Log(relpath+": No room for file on target.", MammothCopyView.LEVEL_NOTICE);
		break;
	    case MammothTelegram.PROT_RTS_FILEDONE:
		// file already exists, do not send
		MammothCopyView.Log(relpath+": File is done on target.", MammothCopyView.LEVEL_NOTICE);
		break;
	    case MammothTelegram.PROT_RTS_PARTDONE:
		// file is partially done,
		// we will have to start with chunk #result.data[1] + 1
		r = Integer.parseInt(result.data[1]) + 1;
		MammothCopyView.Log(relpath+": File is partially done. Starting from chunk #"+r, MammothCopyView.LEVEL_NOTICE);
		break;
	    case MammothTelegram.PROT_RTS_FILENEW:
		// file is new on target, start with chunk 0
		MammothCopyView.Log(relpath+": File is new on target.", MammothCopyView.LEVEL_NOTICE);
		r = 0;
		break;
	    default:
		// unknown status. skip file
		MammothCopyView.Log(relpath+": Unknown status received: "+result, MammothCopyView.LEVEL_WARNING);
		break;
	}

	return r;
    }

    private void SendChunks(MammothFile file, int startchunk) throws IOException {
	RandomAccessFile str = new RandomAccessFile(file, "r");
	byte[] chunk;

	int chunk_to_send = startchunk;
	int tries = socket.chunktries;
	int totalchunks = (int)file.length()/socket.chunksize;

	boolean all_ok = true;
	while ((chunk = GetChunk(str, chunk_to_send)) != null && all_ok == true) {
	    // send the chunk
	    String hash = Generic.toHash(chunk, socket.md);
	    String tel = "CHUNK:"+MammothTelegram.PROT_CHUNK_CHUNK+":"+socket.getRelativePath(file)+":"+chunk_to_send+":"+hash;

    	    MammothCopyView.Log("[telegram] "+tel, MammothCopyView.LEVEL_DEBUG);
	    socket.out.println(tel);

	    MammothCopyView.Log("[data] Going to send chunk #"+chunk_to_send+", retries "+tries+": "+chunk.length+" bytes.", MammothCopyView.LEVEL_DEBUG);

	    socket.sock.getOutputStream().write(chunk, 0, chunk.length);

	    MammothCopyView.Log("[data] Chunk #"+chunk_to_send+" sent, waiting for reply.", MammothCopyView.LEVEL_DEBUG);
	    // wait for chunk reply
	    String line = socket.in.readLine();
	    MammothTelegram result = new MammothTelegram(line);
	    MammothCopyView.Log("[telegram] "+line, MammothCopyView.LEVEL_DEBUG);

	    switch (result.status) {
		case MammothTelegram.PROT_CHUNK_DONE:
		    int percentage = ((chunk_to_send+1) * 100 / (totalchunks+1));
		    String s = "Chunk "+(chunk_to_send+1)+"/"+(totalchunks+1)+" ("+percentage+"%)";
		    MammothCopyView.SetSubProgressBar(s, percentage);

		    MammothCopyView.Log("Chunk #"+chunk_to_send+" sent successfully.", MammothCopyView.LEVEL_NOTICE);
		    chunk_to_send++;
		    tries = socket.chunktries;

		    break;
		case MammothTelegram.PROT_CHUNK_NOMATCH:
		    if (tries == 0) {
			// bail out
			MammothCopyView.Log("Tried "+socket.chunktries+" time(s) to send chunk #"+chunk_to_send+". We are done for this file.", MammothCopyView.LEVEL_NOTICE);
			all_ok = false;
		    } else {
			tries--;
			MammothCopyView.Log("Chunk #"+chunk_to_send+" had a hash mismatch. Retries "+tries, MammothCopyView.LEVEL_NOTICE);
		    }
		    break;
		case MammothTelegram.PROT_CHUNK_DISKFULL:
		    MammothCopyView.Log("Target disk is full. We are done for this file: "+file.getName(), MammothCopyView.LEVEL_NOTICE);
		    all_ok = false;
		    break;
		default:
		    MammothCopyView.Log("Got a strange response: "+line, MammothCopyView.LEVEL_ERROR);
		    break;
	    }
//	    chunk_to_send++;
	}
    }

    private byte[] GetChunk(RandomAccessFile file, int chunkno) {
	byte[] tmpchunk = new byte[socket.chunksize];
	byte[] chunk = null;

	try {
	    long start = chunkno * socket.chunksize;
	    MammothCopyView.Log("Chunk #"+chunkno+" starts at "+start+", file length="+file.length(), MammothCopyView.LEVEL_DEBUG);

	    if (start < file.length()) {
		file.seek(start);

		// always read exactly chunksize bytes
		int bytesread = file.read(tmpchunk, 0, socket.chunksize);

		chunk = new byte[bytesread];
		chunk = Arrays.copyOf(tmpchunk, bytesread);
		MammothCopyView.Log("Chunk #"+chunkno+" starts at "+start+" and is "+bytesread+" long", MammothCopyView.LEVEL_DEBUG);
	    } else {
		MammothCopyView.Log("Chunk #"+chunkno+" is beyond last chunk.", MammothCopyView.LEVEL_DEBUG);
		chunk = null;
	    }
	} catch (Exception e) {
	    // something went wrong, assume this file done
	    MammothCopyView.Log("Couldn't fetch chunk #"+chunkno+": "+e, MammothCopyView.LEVEL_ERROR);
	    chunk = null;
	}

	return chunk;
    }


    // run the sender role. note that this function runs inside a thread
    // and should not wait.
    // it should use blocking reads to avoid a busy-wait loop
    // when in.readline() returns null, the other party disconnected.
    public void Run(MammothSocket _s, MammothTelegram tel) {
	MammothFile file;
	int startchunk = 0;
	socket = _s;

	try {
	    int filetosend = 1;
	    int totalfiles = socket.getNumberOfFiles();
	    while ((file = socket.getNextFile()) != null) {
		int percentage = (filetosend * 100)/ totalfiles;
		String s = "File "+filetosend+" of "+totalfiles+" ("+percentage+"%)";
		MammothCopyView.SetProgressBar(s, percentage);

		MammothCopyView.Log("Sending RTS for file "+filetosend+" of "+totalfiles+": "+file.getAbsolutePath(), MammothCopyView.LEVEL_NOTICE);
		if ((startchunk = SendRTS(file)) != SENDER_NOSEND) {
		    MammothCopyView.Log("RTS sent. Sending chunks.", MammothCopyView.LEVEL_NOTICE);
		    SendChunks(file, startchunk);
		    MammothCopyView.Log("File done: "+file.getAbsolutePath(), MammothCopyView.LEVEL_DEBUG);
		}
		filetosend++;
	    }
	} catch (SocketException e) {
	    MammothCopyView.Log("Disconnected while sending file(s): "+e, MammothCopyView.LEVEL_WARNING);
	} catch (IOException e) {
	    MammothCopyView.Log("IOEXception in sending files: "+e, MammothCopyView.LEVEL_ERROR);
	}

	// We are done. Disconnect
	MammothCopyView.Log("Sending done. Disconnect.", MammothCopyView.LEVEL_NOTICE);
	socket.Cancel();
    }
}
