/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package mammothcopy;

import java.net.*;
import java.io.*;
import java.security.*;
import java.util.*;

/**
 *
 * @author bart
 *
 * MammothSocket implements the MammothCopy protocol
 * Can either be called by MammothServer or MammothClient
 */
public class MammothSocket extends Thread {
    Socket sock;
    String filetosend;
    
    // we need the root, to create the relative path of files 
    // we are going to send.
    String filesroot;
    public boolean connected;
    int state;
    int role;
    int chunksize;

    int chunktries;
    int filepointer;

    static final int STATE_IDLE = 0;
    static final int STATE_HANDSHAKE = 1;
    static final int STATE_CHUNK = 2;

    MessageDigest md;

    // we use sock.get(In|Out)putStream to send/receive the binary chunks
    // these are for the telegrams; got this from
    // http://java.sun.com/docs/books/tutorial/networking/sockets/clientServer.html
    // (so it should be right ;-))
    PrintWriter out;		    // this has a convenient println
    BufferedReader in;		    // this has a convenient readLine

    Vector<String> filestosend;	    // holds 'String' objects
    String target;

    MammothSocket(Socket s) {
        sock = s;
        state = STATE_IDLE;
        role = MammothRole.ROLE_NONE;
	connected = true;
	filepointer = 0;
	filestosend = new Vector<String>();
	filesroot = "";

	// we try 3 times per chunk before aborting the transfer
	chunktries = 3;
    }

    public MammothFile getNextFile() {
	MammothFile r;
	try {
	    r = new MammothFile(filestosend.get(filepointer++), md);
	} catch (ArrayIndexOutOfBoundsException e) {
	    // we are done.
	    r = null;
	}
	return r;
    }

    public int getNumberOfFiles() {
	return filestosend.size();
    }


    public String getRelativePath(MammothFile  file) {
	return file.getAbsolutePath().substring(filesroot.length()+1);
    }

    // one run is one transfer.
    @Override public void run() {
        MammothCopyView.Log("Started socket, let's do a transfer! "+sock.getRemoteSocketAddress().toString(), MammothCopyView.LEVEL_NOTICE);
	MammothRole mrole = null;

        try {
            in = new BufferedReader(new InputStreamReader(sock.getInputStream()));
            out = new PrintWriter(sock.getOutputStream(), true);
	    MammothTelegram result = null;
            while (connected) {
		// 1. Figure out role. This is the transfer setup stage
                while (role == MammothRole.ROLE_NONE) {
                    String line = in.readLine();
                    result = new MammothTelegram(line);

                    switch (result.status) {
                        case MammothTelegram.PROT_HS_REQ:
                            MammothCopyView.Log("Handshake REQ received. We will be receiver.", MammothCopyView.LEVEL_NOTICE);
                            role = MammothRole.ROLE_RECEIVER;
                            break;
                        case MammothTelegram.PROT_HS_ACK:
                            MammothCopyView.Log("Handshake ACK received. We will be sender.", MammothCopyView.LEVEL_NOTICE);
                            role = MammothRole.ROLE_SENDER;
                            break;
			case MammothTelegram.PROT_HS_NAK:
			    MammothCopyView.Log("Handshake NAK received. Stop.", MammothCopyView.LEVEL_NOTICE);
			    role = MammothRole.ROLE_ABORT;
			    break;
			case MammothTelegram.PROT_ERROR:
			    MammothCopyView.Log("Incorrect telegram received. Stop.", MammothCopyView.LEVEL_WARNING);
			    role = MammothRole.ROLE_ABORT;
			    break;
			case MammothTelegram.PROT_EMPTY:
			    MammothCopyView.Log("Other side disconnected. Stop.", MammothCopyView.LEVEL_WARNING);
			    role = MammothRole.ROLE_ABORT;
			    break;
                        default:
                            MammothCopyView.Log("Received unknown data: "+line+" "+result, MammothCopyView.LEVEL_ERROR);
                            break;
                    } 
                }

		// 2. We know our role. Create that role.
		switch (role) {
		    case MammothRole.ROLE_SENDER:
			mrole = new MammothSender();
			break;
		    case MammothRole.ROLE_RECEIVER:
			mrole = new MammothReceiver();
			break;
		    case MammothRole.ROLE_ABORT:
			MammothCopyView.Log("Connection aborted.", MammothCopyView.LEVEL_ERROR);
			Cancel();
			break;
		    default:
			// we got out of the role checker without a role. Abort.
			MammothCopyView.Log("Created unknown role: "+role+". Stop. (this shouldn't happen)", MammothCopyView.LEVEL_ERROR);
			Cancel();
			break;
		}

		// 3. Run the role
		if (mrole != null) {
		    MammothCopyView.Log("Running role: "+role, MammothCopyView.LEVEL_DEBUG);
		    mrole.Run(this, result);
		}
            }
        } catch (IOException e) {
            MammothCopyView.Log("Problem in setting up: "+e.toString(), MammothCopyView.LEVEL_ERROR);
        }

        MammothCopyView.Log("Stopped socket."+sock.getRemoteSocketAddress().toString(), MammothCopyView.LEVEL_NOTICE);
    }

    public int SetFileToSend(String f) {
	filestosend = new Vector<String>();

	try {
	    File ff = new File(f);
	    filesroot = ff.getParent();
	} catch (Exception e) {
	    MammothCopyView.Log("Could not find file root: "+e.toString(), MammothCopyView.LEVEL_NOTICE);
	}

	return __SetFileToSend(f);
    }
    // recursion function
    private int __SetFileToSend(String f) {
	File ff = new File(f);

	if (ff.isDirectory()) {
	    // cut off possible separator chars
	    if (f.charAt(f.length()-1) == File.separatorChar) {
		f = f.substring(0, f.length()-1);
	    }
	    String[] l = ff.list();
	    for (int p = 0; p < l.length; p++) {
		__SetFileToSend(f+File.separator+l[p]);
	    }
	} else {
	    if (ff.isFile()) {
		filestosend.add(f);
	    }
	}

	MammothCopyView.Log("File tree loaded: "+filestosend.size()+" file(s)", MammothCopyView.LEVEL_NOTICE);

	return filestosend.size();
    }

    public void setTarget(String s) {
	// just set it, we check it on a received HS telegram
	target = s;
    }

    public void Cancel() {
	try {
	    sock.close();
	    connected = false;
	} catch (IOException e) {
	    MammothCopyView.Log("IOException in socket.close():"+e, MammothCopyView.LEVEL_ERROR);
        }
    }

    public void Go(String h, int _chunksize) {
        // this starts the negotiating and sending
        MammothCopyView.Log("Starting handshake, figuring out roles.", MammothCopyView.LEVEL_NOTICE);

        try {
	    chunksize = _chunksize;
	    md = MessageDigest.getInstance(h);
            out.println("HS:"+MammothTelegram.PROT_HS_REQ+":"+h+":"+chunksize);
	} catch (NoSuchAlgorithmException e) {
	    MammothCopyView.Log("Digest algorithm '"+h+"' not supported: "+e, MammothCopyView.LEVEL_WARNING);
        } catch (Exception e) {
            MammothCopyView.Log(e.toString(), MammothCopyView.LEVEL_ERROR);
        }
    }
}
